@@ -64,6 +64,11 @@ impl PrimalityTestConfig {
         }
     }
 
+    /// Create a configuration with the known stongest primality test
+    pub fn strict() -> Self {
+        Self::bpsw()
+    }
+
     /// Create a configuration for Baillie-PSW test (base 2 SPRP test + SLPRP test)
     pub fn bpsw() -> Self {
         Self {
@@ -104,6 +109,8 @@ pub struct FactorizationConfig {
 }
 
 impl FactorizationConfig {
+    /// Create a defalt primality testing configuration. This config will factorize
+    /// most integers within decent time
     pub fn default() -> Self {
         Self {
             prime_test_config: PrimalityTestConfig::default(),
@@ -114,6 +121,13 @@ impl FactorizationConfig {
             pp1_trials: 0,
         }
     }
+
+    /// Same as the default configuration but with strict primality check
+    pub fn strict() -> Self {
+        let mut config = Self::default();
+        config.prime_test_config = PrimalityTestConfig::strict();
+        config
+    }
 }
 
 // FIXME: backport to num_integer (see https://github.com/rust-num/num-traits/issues/233)
@@ -144,42 +158,18 @@ pub trait ExactRoots: Roots + Pow<u32, Output = Self> + Clone {
     }
 }
 
-/// This trait describes modular arithmetic on a integer
-pub trait ModInt<Rhs = Self, Modulus = Self> {
-    type Output;
+// TODO (v0.2): implement quick div_exact (which might be useful in various functions)
+// REF: GMP `mpz_divexact`
+//      FLINT `fmpz_divexact`
+//      factor.c `divexact_21`
 
-    /// Return (self + rhs) % m
-    fn addm(self, rhs: Rhs, m: Modulus) -> Self::Output;
-
-    /// Return (self + rhs) % m
-    fn subm(self, rhs: Rhs, m: Modulus) -> Self::Output;
-
-    /// Return (self * rhs) % m
-    fn mulm(self, rhs: Rhs, m: Modulus) -> Self::Output;
-
-    /// Return (self ^ exp) % m
-    fn powm(self, exp: Rhs, m: Modulus) -> Self::Output;
-
-    /// Return (-self) % m and make sure the result is in range [0,m)
-    fn negm(self, m: Modulus) -> Self::Output;
-
-    /// Calculate inverse module (x such that self*x = 1 mod m)
-    fn invm(self, m: Modulus) -> Option<Self::Output>
-    where
-        Self: Sized;
-
-    /// Calculate Jacobi Symbol (a|n), where a is self
-    fn jacobi(self, n: Modulus) -> i8;
-
-    // TODO: Calculate Kronecker Symbol (a|n), where a is self
-    // fn kronecker(self, n: Modulus) -> i8;
-
-    // TODO: Modular sqrt
-    // fn sqrtm(self, m: Modulus);
-}
+// TODO: implement quick is_x_power (specifically is_235_power)
+// REF: PARI/GP `Z_ispowerall`, `is_357_power`
+//      FLINT `n_is_perfect_power235`, `fmpz_is_perfect_power`
+//      GMP `mpz_perfect_power_p`
 
 /// This trait represents a general data structure that stores primes.
-/// 
+///
 /// It's recommended to store at least a bunch of small primes in the buffer
 /// to make some of the algorithms more efficient.
 pub trait PrimeBuffer<'a> {
@@ -203,7 +193,7 @@ pub trait PrimeBuffer<'a> {
 }
 
 /// This trait implements various primality testing algorithms
-/// 
+///
 /// Reference:
 /// - <http://ntheory.org/pseudoprimes.html>
 pub trait PrimalityUtils: Integer + Clone {
